#!/usr/bin/perl
#----------------------------------------
# Horn's INF model : solve with Euler's method
#
# dv/dt = -kv+a+cmv+mI
# dm/dt = -m+H(m-v)
#----------------------------------------
$usage = <<'EOF' ;
usage: hoen.pl [-h] [-d <duration>] [<k> <a> <c> <I>]
	-h : help
	<duration> : how many msec to simulate

	dv/dt = -kv+a+cmv+mI
	dm/dt = -m+H(m-v)

	Default values:
	k = 0.45, a=-0.09, c=0.35, I=0.29
EOF

#----------------------------------------
# Print help message
#----------------------------------------
require "getopts.pl";
&Getopts('hd:');

if ($opt_h) {
	print $usage;
	exit(0);
}

#----------------------------------------
# Set default parameters and input
#----------------------------------------
if ($#ARGV!=3) {
	print STDERR "horn.pl : k = 0.45, a=-0.09, c=0.35, I=0.29\n";
	$parm_k = 0.45;
	$parm_a = -0.09;
	$parm_c = 0.35;
	$parm_I = 0.29;
} else {
	$parm_k = $ARGV[0];
	$parm_a = $ARGV[1];
	$parm_c = $ARGV[2];
	$parm_I = $ARGV[3];
}

$duration = ($opt_d)?$opt_d:1000;

#----------------------------------------
# Integrate : step size $h = 0.01;
#----------------------------------------
$t = 0;
$h = 1/10.0;

#----------------------------------------
# Initial values
#----------------------------------------
@vec = ( 0.0, 1.0 );

#----------------------------------------
# Define the differential equations
#----------------------------------------
# dv/dt = -kv+a+cmv+mI
# dm/dt = -m+H(m-v)
#----------------------------------------
@func = ( '-$parm_k*$oldVec[0] + $parm_a + $parm_c*$oldVec[1]*$oldVec[0] 
		+ $oldVec[1]*$parm_I',
	  '-$oldVec[1]+(($oldVec[1]-$oldVec[0]>0.0)?1.0:0.0)');


#----------------------------------------
# Run!
#----------------------------------------
$time = 0.0;
$iters = $duration / $h;
for ($t = 0; $t<$iters; ++$t) {
	@oldVec =  @vec;
	print $time;
	for ($k=0; $k<=$#vec; ++$k) {
		$vec[$k] = $oldVec[$k] + $h*(eval $func[$k]);
		print " $vec[$k]";
	}
   	print "\n";
	$time += $h;

}
