//----------------------------------------------------------------------------
// 
//	file: geninput.h
// 
//	function: generate input 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Sep 11 15:05:06 CDT 1998
// 
//	$Revision: 1.6 $ $Date: 2001/11/30 20:24:15 $
//	$Log: geninput.h,v $
//	Revision 1.6  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.5  1999/02/09 07:42:25  yschoe
//	updated sine grating input
//
//	Revision 1.4  1999/01/05 09:20:50  yschoe
//	made gaussian() available as a static function.
//	wrote sine_grating routine -- must make it static too.
//
//	Revision 1.3  1998/11/17 12:42:30  yschoe
//	added ppm read routine.
//
//	Revision 1.2  1998/10/05 19:48:46  yschoe
//	separated gaussian routine from constructor(?)
//
//	Revision 1.1  1998/10/04 05:25:44  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#ifndef __GENINPUT_H
#define __GENINPUT_H

#include <iostream.h>
#include <math.h>
#include "real.h"
#include "util.h"
#include "matrix.h"

// supported types of input
enum INP_TYPE { INP_SQR, INP_ORI_BAR, INP_BLOB, INP_ORI_BLOB, INP_IMG, INP_SIN, INP_BLOBS,INP_CURV};

// normalization options : by sum, by sqrt of sum of sqr, or no normalization
enum INP_NORM { INP_NORM_SUM, INP_NORM_SQRT, INP_NORM_NONE };

// theta : orientation 
//             90
//        180 _|_ 0
//             |
//            270                 

class geninput {

  INP_TYPE 	inp_type;	// input type
  INP_NORM	norm_type;	// input type
  REAL 		angle;		// degrees
  int  		cols, rows;	// size of input matrix
  int  		cx, cy;  	// x and y location of input
  REAL 		sigmax, sigmay; // for gaussian blobs and oriented blobs
  int		radius;		// for square
  int		width, height;	// for oriented bars
  int		imgcount;

public:

  REAL **input;		// Where the actual activity is saved.
			// This is just a pointer to some place malloced 
			// in other classes.
  matrix<REAL> **images; // preloaded images from file: images[index]->mat

 	// Constructor
  	geninput(REAL **input, int cols, int rows, INP_TYPE type,INP_NORM norm,
		char *imgfilename = NULL, int imgcount = 0);

	// Destructor
       ~geninput();

  void set_type(INP_TYPE t) { inp_type = t; }

  // Clear input matrix
  void	clear_input();
  void  clear_rows();
  void  clear_rows(int start, int end); 

  // add input to input matrix
  void	add_input(int cx, int cy, int radius);
  void	add_input(int cx, int cy, int width, int height, REAL angle);
  void	add_input(int cx, int cy, REAL sigma);
  void	add_input(int cx, int cy, REAL sigmax, REAL sigmay, REAL angle);
  void	add_input(int cx, int cy, int freq, REAL phase, REAL angle);
  void	add_input(int x,  int y, REAL sigma, REAL angle);
  void  add_input(int imgidx);

  // clear and generate random input of default input type.
  // void	random_input(); 

  static void  gaussian(REAL **input, int cols, int rows, int cx, int cy, 
		REAL sigmax, REAL sigmay, REAL angle);
  static void  sine_grating(REAL **input, int cols, int rows, int cx, int cy,
		int freq, REAL phase, REAL angle);

protected:

  inline REAL norm_factor(REAL sum) { // normalization factor
	if (norm_type == INP_NORM_SUM) {
        	return sum;
	} else {
		return sqrt(sum);
 	}
  };
  inline REAL norm_sum(REAL sum) { // when summing input activity
	if (norm_type == INP_NORM_SUM) {
        	return sum;                    
	} else {
		return sum*sum;
 	}
  };

  // normalize weights 
  void   normalize(int lowx, int highx, int lowy, int highy, REAL sum);

  // read input from file
  void  read_input(char *filename);

  // generic gaussian distribution generator 
  void  gaussian(REAL **input, int cx, int cy, 
		REAL sigmax, REAL sigmay, REAL angle);
  void  sine_grating(REAL **input, int cx, int cy,
		int freq, REAL phase, REAL angle);

}; 

#endif
