//----------------------------------------------------------------------------
// 
//	file: orpreftest.cc
// 
//	function: test or pref calc routine in netlayer class 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sun Jan 10 00:13:35 CST 1999
//
//	orpreftest2: Jim's vector sum method
//		- 0, 60 and 120 degrees --> x2
//		- calculate max response for each angle
//		- with (rho,phi) where rho == max response and phi == angle
//		  as vectors, sum the three vectors to get a vector sum.
//		- the length of this vector is the selectivity, and
//		  the angle is the or preference
//		
// 
//	$Revision: 1.1 $ $Date: 1999/02/09 07:56:44 $
//	$Log: orpreftest.cc,v $
//	Revision 1.1  1999/02/09 07:56:44  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------


#include <window.h>
#include <unistd.h>
#include <math.h>
#include <iostream.h>
#include <stream.h>
#include <nnview/real.h>
#include <nnview/matrix_window.h>
#include <nnview/plot_window.h>
#include <nnview/tool_window.h>
#include <nnview/util.h>
#include <nnview/geninput.h>
#include <nnview/disputil_window.h>

#define RADIUS(x,y,cx,cy) (sqrt(((x)-(cx))*((x)-(cx))+((y)-(cy))*((y)-(cy)))+0.5)

main_window     *mw;
matrix_window   *matw;
matrix_window   *matw2;
matrix_window   *matw3;
callback_button *runit;
callback_button *clearit;
geninput 	*input;
geninput 	*match;

REAL **mat;
REAL **ref;
REAL **mat3;

REAL gsigmax;
REAL gsigmay;
int locx, locy;

int N = 25;


//----------------------------------------------------------------------------
//	calc_single_or_pref() 
//	  - calculate or pref : copied from netlayer.cc
//----------------------------------------------------------------------------
REAL calc_single_or_pref(

	REAL **ref_pattern, 
	REAL **actual_pattern,
	int x_size, int y_size, 
	REAL sigmax, REAL sigmay) {

  // for 0 to 180 degrees, place gaussian blob on the COG of that
  // neuron's afferent weight matrix, and calculate the maximally
  // responding one, and return it in 0.0(0deg)-1.0(180deg) scale.
  int samples = 18;
  int step    = samples/6;
  REAL max_or_resp[samples];

  int location;

  REAL mult = 180.0/(REAL)samples;
  REAL max_resp = -99999.9;
  int  max_idx  = 0;

  for (int i=0; i<samples; ++i) {
	max_or_resp[i] = 0.0;
  }

  for (int k=0; k<samples; k += step) { // for 0 - 180 degree rotation

    int diag = (int)(sqrt(x_size*x_size+y_size*y_size)*1.5);
    for (int p=0; p<MAX(x_size,y_size); ++p) { // for a complete sweep from left to right
      // generate gaussian
      for (int i=0; i<x_size; ++i)
          for (int j=0; j<y_size; ++j)
                ref_pattern[i][j] = 0.0;

      if (k<samples/2) {
                geninput::gaussian(ref_pattern, x_size, y_size,
                        (int)x_size-p,p, 
			(REAL)x_size*2.0, (REAL)5.0, (REAL)(k * mult));
      } else {
                geninput::gaussian(ref_pattern, x_size, y_size,
                        x_size-p, (int)y_size-p,
			(REAL)x_size*2.0, (REAL)5.0, (REAL)(k * mult));
      }

/*
      matw2->redraw();
      XFlush(display);
*/

      // calc dotproduct (as a vector)
      REAL dot = 0.0;
      for (int i=0; i<x_size; ++i)
          for (int j=0; j<y_size; ++j)
	      if (RADIUS(i,j,N/2,N/2)<N/2)	
              	dot += ref_pattern[i][j]*actual_pattern[i][j];
      //debug: cout << dot;

      if (dot>=max_or_resp[k]) {
          max_or_resp[k] = max_resp = dot;
	  max_idx  = k;
          //debug: cout << " " << k << " * "  << max_resp << endl;
	  match->clear_input();

/*
          if (k<samples/2) {
		match->add_input(
			(int)x_size-p,y_size,
			(REAL)(x_size*2.0), (REAL)1.0, (REAL)(k * mult));
          } else {
		match->add_input(
			x_size-p, (int)y_size-p,
			(REAL)(x_size*2.0), (REAL)1.0, (REAL)(k * mult));
          }
      	  matw3->redraw();
          XFlush(display);
*/
      } else {
          //debug: cout << endl;
      }
    }
    //debug: cout << "-------" << endl;

  }

  /*
  max_or_resp[0] = 10.0;
  max_or_resp[step] = 20.0;
  max_or_resp[step*2] = 10.0;
  */

  // Calculate vector sum
  REAL vector_sum = 0.0;
  REAL sum = 0.0;
  REAL cart_x = 0.0;
  REAL cart_y = 0.0;  
  REAL comp = 0.0;
  REAL selectivity = 0.0;
  int  count=0;
  for (int i=0; i<samples; i+=step) {
	cart_x += max_or_resp[i] * cos(2.0*i*10.0*3.141592/180.0);
	cart_y += max_or_resp[i] * sin(2.0*i*10.0*3.141592/180.0);
	count ++;
  }

  // cout << "count = " << count << endl;

  cart_x = cart_x;
  cart_y = cart_y;

  vector_sum = 180.0*atan2(cart_y,cart_x)/3.141592/2.0; 
  selectivity = sqrt(cart_y*cart_y+cart_x*cart_x);
  cout << "selectivity " << selectivity << endl;
  if (vector_sum < 0.0) vector_sum += 180.0;

  // cout << "vector_sum (" << cart_x << ","
  //	<< cart_y << ") " << "angle= " << vector_sum << endl;

/*
   match->clear_input();
  if (vector_sum<samples*10/2) {
	match->add_input(
		x_size/2,y_size/2,
		(REAL)(x_size*2.0), (REAL)1.0, (REAL)(vector_sum));
  } else {
	match->add_input(
		x_size/2,y_size/2,
		(REAL)(x_size*2.0), (REAL)1.0, (REAL)(vector_sum));
  }
  matw3->redraw();
  XFlush(display);
*/

  //debug: cout << "------------------" << endl;

  return (vector_sum/180.0); // 0.0 == 0 degree, 1.0 == 180 degrees

}


// the actual action takes place here.
void run() {

  //static int i, j;
  static int count;
  for (int i=0; i<180; ++i) {
  	input->clear_input();
  	match->clear_input();
 	REAL sigmax = gsigmax; //(REAL)(drand48()*2+1);
	REAL sigmay = gsigmay; //REAL sigmay = (REAL)(drand48()*8+1);
	// REAL angle  = (count*10)%360; //REAL angle  = (REAL)(drand48()*360.0);
	REAL angle  = (count)%360; //REAL angle  = (REAL)(drand48()*360.0);
  	cout << "sigmax = " << sigmax 
	     << " sigmay = " << sigmay
	     << " angle = " << angle << endl;
  	input->add_input(locx,locy,
			sigmax, sigmay, angle);
			// 10,10,
			// 3,(REAL)0.0,0.0);//(REAL)angle);	
			//(int)(drand48()*N),(int)(drand48()*N),
			//(REAL)(drand48()*8+3),(REAL)(drand48()*16+3),
			//(REAL)(drand48()*360.0));
  /*	input->add_input((int)(drand48()*N),(int)(drand48()*N),
			(REAL)(drand48()*8));
  */
        for (int k=0; k<N; ++k)
            for (int l=0; l<N; ++l)
	        if (RADIUS(k,l,N/2,N/2)>N/2)	
			input->input[k][l] = 0.0;

  	matw->redraw();
	REAL ori=calc_single_or_pref(ref,input->input,N,N,sigmax,sigmay)*180.0 ;
	cout << "orientation = " << ori << endl;

	cout << i << " " << (((i-ori)<-90)?(180-i-ori):(i-ori)) << " error " << endl;
  	XFlush(display);
  	count ++;
  }
  //cout << "run\n";
}

// clear mat
void clear() {
  disputil_window *dw = new disputil_window("disputil",mat,N,N,20);
  // input->clear_input();
  matw->redraw();
  XFlush(display);
  cout << "clean\n";
}

// Main
int main(int argc, char *argv[]) {

  if (argc != 5) {
  	cout << "usage: orpreftest <sigmax> <sigmay> <locx> <locy>" << endl;
	cout << " 	sigmax : length of gaussian bar " << endl;
	cout << " 	sigmay : width of gaussian bar " << endl;
	cout << " 	locx : x location of gaussian bar " << endl;
	cout << " 	locy : y location of gaussian bar " << endl;
	exit(0);
  }
  mat = new REAL* [N];
  for (int i=0; i<N; ++i) mat[i] = new REAL [N];
  mat3 = new REAL* [N];
  for (int i=0; i<N; ++i) mat3[i] = new REAL [N];
  ref = new REAL* [N];
  for (int i=0; i<N; ++i) ref[i] = new REAL [N];

  gsigmax = (REAL)atoi(argv[1]);
  gsigmay = (REAL)atoi(argv[2]);
  locx = atoi(argv[3]);
  locy = atoi(argv[4]);

  // setup input class
  input = new geninput(mat,N,N,INP_SQR,INP_NORM_SQRT);
  match = new geninput(mat3,N,N,INP_SQR,INP_NORM_SQRT);
  input->clear_input();

  // setup the windows
  mw = new main_window(argv[0],722,260);

  matw = new matrix_window(*mw,240,240,0,0,0,60,COLOR,N,N,mat);
  matw2 = new matrix_window(*mw,240,240,241,0,0,60,COLOR,N,N,ref);
  matw3 = new matrix_window(*mw,240,240,482,0,0,60,COLOR,N,N,mat3);

  runit = new callback_button(*mw,"Run",run,
                mw->width/4,20,mw->width/2,mw->height-20);

  clearit = new callback_button(*mw,"Clean",clear,
                mw->width/4,20,mw->width*1/4,mw->height-20);

   
  // add quit button
  new quit_button(*mw,mw->width/4,20,mw->width*3/4,mw->height-20);

  // start main loop
  mw->main_loop();

}
  
