//----------------------------------------------------------------------------
// 
//	file: drawline.cc
// 
//	function: test a simple line drawing algorithm 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Mon Feb 15 23:44:19 CST 1999
// 
//	$Revision: 1.2 $ $Date: 2002/06/20 22:35:52 $
//	$Log: drawline.cc,v $
//	Revision 1.2  2002/06/20 22:35:52  yschoe
//	*** empty log message ***
//
//	Revision 1.1  1999/02/16 05:44:24  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#include <iostream.h>
#include <string.h>
#include <math.h>
#include <nnview/matrix.h>

enum DRAW_MODE {DRAW_RAND, DRAW_LINE};

void usage(int n) {
  cout << "usage: drawline (<n> <m>) | (<I:n> <R:a> <R:b>)" << endl;
  cout << "    <n> : size of matrix - <n>x<n> " << endl;
  cout << "    <m> : line number divisor " << endl; 
  cout << "    draw y = <a>x + <b> " << endl;
  exit(n);
}

float myrand(int max) {

  return max*rand()/(RAND_MAX+1.0);

}

int main (int argc, char *argv[]) {

  DRAW_MODE selection;
  float a, b;
  int   n;
  int	divisor;

  if (argc==3) {
	selection = DRAW_RAND;
	divisor   = atoi(argv[2]);
	n = atoi(argv[1]);
  } else if (argc!=4) {
	usage(2);
  } else {
	selection = DRAW_LINE;
	n = atoi(argv[1]);
  }

  matrix<float> *mat = new matrix<float>(n,n);
  
  switch(selection) {

  case DRAW_LINE:

	  a = atof(argv[2]);
	  b = atof(argv[3]);

	  for (int i=0; i<n; ++i) {
		for (int j=0; j<n; ++j) {
			if (int(a*i+b)==j || int((j-b)/a)==i) {
				mat->mat[i][j] = 1;	
			} else {
				mat->mat[i][j] = 0;
			}
		}
	  }

	  break;
	  
  case DRAW_RAND:
	  for (int i=0; i<n; ++i)
	     for (int j=0; j<n; ++j) 
		mat->mat[i][j] = 0;

	  int m = n/divisor;
	  float logmax = log(m);
	  for (int k=1; k<=m; ++k) {
		  a = tan(90.0/m*k*M_PI/180.0);
		  b = 0;
		  //float lineval = log((float)k)/logmax;	
		  float lineval = k/(float)m;
		  //float lineval = 1.0;
		  for (int i=0; i<n; ++i) {
			for (int j=0; j<n; ++j) {
				if (int(a*i+b)==j || int((j-b)/a)==i) {
					mat->mat[i][j] = lineval;
				} 
			}
		  }
	  }
	  break;

  } // end switch

  mat->print_matrix();

}
