## usage: [nS,Eimg] = convedgespacemat(Img, kernelsize, gridsize,the##                                                                          ta)
##
##	Img: p x q matrix (grayscale image)
##
##		Img: 2D matrix with grayscale image
##		kernelsize: width (==height) of convolution kernel
##		gridsize:   sampling grid size
##		theta:      gabor filter orientation
##
## Returns a n x m matrix S where 
##	n = p / gridsize
##	m = q / gridsize
##	and each S_ij is the spatial frequency of the subimage patch.
##
## First, convolve with DoG filter, and then convolve with gabor
## filters of spatial frequencies  1, 2, ... , 10. Cut the generated images
## into patches and find the maximum value and the index	
##
##	Usage example:
##		[nS,Smax,Eimg] = convedgespacemat(imagemat,9,3,0);
##
##		both DoG and gabor filters are  9 x 9, and
##		Gabor orientation is 0.
##		final grid sampling size is 3 x 3.
##
## $Id: convedgemat.m,v 1.2 2002/03/09 17:31:27 yschoe Exp yschoe $

function [nS,Smax,Eimg] = convedgespacemat(Img, kernelsize, gridsize,theta)

	# get image size
	[oldp, oldq] = size(Img);

	# adjust grid size to have the conv kernels
	# centered at the middle: i.e. make the gridsize an odd number.
	if ( rem(kernelsize,2) == 0 ) 
		kernelsize ++;
	endif

	DoG = dog(kernelsize);

        # convolve with DoG, and only use the real part
	Eimg = real(conv2(Img,DoG,"same"));	

	
        # calculate new image size to make it divisible by gridsize
	p = floor(oldp/gridsize)*gridsize;
	q = floor(oldq/gridsize)*gridsize;

   	ygrids = floor(p/gridsize);
	xgrids = floor(q/gridsize);

	#initialize matrices
        V = zeros(ygrids,xgrids).-999999;
	nS = zeros(ygrids,xgrids);
	
	Vmax=zeros(oldp,oldq);
        Smax=zeros(oldp,oldq);
        theta=0; 
	#g = gabor(kernelsize,kernelsize,"sin",theta,1);
        #g2 = gabor(kernelsize,kernelsize,"cos",theta,1);

        #Vmax=real(conv2(Eimg,g,"same")).^2+real(conv2(Eimg,g2,"same")).^2;
	Smax=Smax+1;
       
        krange=4;
 
        #find Vmax and Smax iteratively
        k=1;
        oldk=0;
        while (theta<=180)
        #k=1;
        while (k<=oldk+krange)
             
             g = gabor(kernelsize,kernelsize,"sin",theta,k-oldk);
             g2 = gabor(kernelsize,kernelsize,"cos",theta,k-oldk);
           vtmp= real(conv2(Eimg,g,"same")).^2+real(conv2(Eimg,g2,"same")).^2;
	     #vtmp=real(conv2(Eimg,g,"same"));
              Vmax=max(Vmax,vtmp);
             Smax=max(Smax,k*(sign(vtmp-Vmax)+1));
             k=k+1;
           
        endwhile
	oldk=k-1;
        theta=theta+5;
        endwhile

        #cut Vmax and Smax into grids and calculate V and nS
 	for j=1:ygrids
          for i=1:xgrids
		stmp1 = vec(Smax(1+((j-1)*gridsize):(j*gridsize),
			        1+((i-1)*gridsize):(i*gridsize)));
		# idx contains the index of the max within that subimage
		# block.
  	        [nS(j,i),idx]=max(stmp1);
	#	stmp = vec(Smax(1+((j-1)*gridsize):(j*gridsize),
	#			1+((i-1)*gridsize):(i*gridsize)));
        #        nS(j,i)=stmp(idx);
          endfor
        endfor

        for i=1:oldp
 	for j=1:oldq
 	if (rem(Smax(i,j),krange)==0)
           Smax(i,j)=krange;
        else
 	   Smax(i,j)=rem(Smax(i,j),krange);
        endif
        endfor
        endfor

        for i=1:xgrids
 	for j=1:ygrids
 	if (rem(nS(i,j),krange)==0)
           nS(i,j)=krange;
        else
 	   nS(i,j)=rem(nS(i,j),krange);
        endif
        endfor
        endfor

endfunction
