## usage: [H] = convhuemat(Img,gridsize)
##
##	Img: p x q matrix (hue values image)
##
##		
##
## Returns a n x m matrix H where 
##	n = p / gridsize
##	m = q / gridsize
##	and each H_ij is the hue value of the subimage patch.
##
##
##	Usage example:
##		[H] = convhuemat(imagemat,5);
##
##
## $Id: convedgemat.m,v 1.2 2002/03/09 17:31:27 yschoe Exp yschoe $

function [H] = convhuemat(Img,gridsize)

	# get image size
	[oldp, oldq] = size(Img);

	Hmax=Img;

        # calculate new image size to make it divisible by gridsize
	p = floor(oldp/gridsize)*gridsize;
	q = floor(oldq/gridsize)*gridsize;

   	ygrids = floor(p/gridsize);
	xgrids = floor(q/gridsize);

	#initialize matrices
	H = zeros(ygrids,xgrids).-999999;
		
	#Hmax=zeros(oldp,oldq);
        
       
	#cut Hmax into grids and calculate H
 	for j=1:ygrids
          for i=1:xgrids
		htmp = vec(Hmax(1+((j-1)*gridsize):(j*gridsize),
			        1+((i-1)*gridsize):(i*gridsize)));
		# idx contains the index of the max within that subimage
		# block.
  	        [H(j,i)]=sum(htmp)/(gridsize*gridsize);
		
          endfor
        endfor

endfunction
