# em.m : a demo for 2D EM algorithm to find the 
# mean of two Gaussians

function [u,X] = em(u1,u2)

	# generate 1D data
	# The datapoints are a mixture of two gaussian distributions
	# with means u1 and u2, with variance 1 for both.

	X1 = randn(1,1000).+u1;	# first gaussian with mean u1
	X2 = randn(1,1000).+u2; # second gaussian with mean u2
	X = [X1,X2];		# just append the two sets of data

	# initial estimate of u1 and u2 : set them to random values
	estu1 = rand*max(X);
	estu2 = rand*max(X);
	
	n = length(X);
	est = [estu1, estu2];

	# loop through E and M steps
	for i=1:30

		# E step:
		p1 = normal_pdf(X,estu1,1);
		p2 = normal_pdf(X,estu2,1);
		E1 = p1./(p1+p2);
		E2 = p2./(p1+p2);

		# plot(p1,"x");
		# plot(p1,"x");

		# M step:
		estu1 = E1*X'/sum(E1);
		estu2 = E2*X'/sum(E2);

		plot([1,2],sort([u1,u2]),[1,2],sort([estu1, estu2]));

		est = [est ; estu1, estu2];

	endfor

	u = est;

endfunction
