# function [c,e] = msfit (dim, x, d, eta, iter, [c]) 
#	fit a cubic polinomial to input->output mapping of x_i --> d_i
#	learning rate : eta
#	c : initial coefficient vector
#	e : final error
#	Example: 
#	[c,e] = msfit(1, (1:10),(1:10)+2*rand(1,10), 0.001,  100000, [ 10 -1]')
function [c,e] = msfit (deg, x, d, eta, iter, c) 

  # init coefficients
  if (nargin<6) 
  	c = rand(deg+1,1)/100.0;
  endif

  # transpose row vectors.
  if (columns(x)>1) 
	x = x';
  endif

  # transpose row vectors.
  if (columns(d)>1) 
	d = d';
  endif

  # transpose row vectors.
  if (columns(c)>1) 
	c = c';
  endif

  # preset common constants
  numinp = length(x);
  disp = 100;
  prev_error = 1000;

  # pre-generate input matrix  (each row is one input)
  inp = (x * ones (1, deg+1)) .^ (ones (numinp, 1) * (0 : deg));

  # repeat epoch

  numcoeff = deg+1;

  cumulerr = [0];
  deltac = zeros(deg+1,1);

  etavec = eta * ((ones(1,deg+1).+10).^(deg:-1:0))';

  for epc = 1:iter

	error = 0;

  	# get output for the whole dataset
  	out = inp*c;

	# calculate error: sqrt of sum of sqr err
	error = error + sqrt(ones(1,numinp)*((out-d).^2));

	error = error / numinp;

	# get deltac
 	#----------------
	#for i=1:numcoeff
	#	 for n=1:numinp
	# 		deltac(i) = deltac(i) + (out(n)-d(n)) *(x(n)**(i-1));
	#	 endfor
	#	 --> inner for loop ==  deltac(i) = (out-d)'*(x.^(i-1));
	#endfor
	# --> make it a matrix operation to save time.
	deltac = ((out-d)' * inp)';

	# update coefficients
	c = c - 2*(etavec.*deltac)/numinp;

	# plot
	if (rem(epc,disp)==0) 
		plot(x,d, "x1" ,x,out, "o2-");
		cumulerr = [ cumulerr, error ];
	endif

	# check for progress
	if (error < 0.01) 
		break;
	endif	

	prev_error = error;
  endfor

  e = cumulerr;
endfunction
