#include <stdio.h>
#include <math.h>
#include "moment.c.numerical"

double kpoints[2000] = {0.0};
double lpoints[2000] = {0.0};
double diff[2000] = {0.0};

#define ITMAX 100
#define EPS 3.0e-7

double betacf(a,b,x)
double a,b,x;
{
	double qap,qam,qab,em,tem,d;
	double bz,bm=1.0,bp,bpp;
	double az=1.0,am=1.0,ap,app,aold;
	int m;

	qab=a+b;
	qap=a+1.0;
	qam=a-1.0;
	bz=1.0-qab*x/qap;
	for (m=1;m<=ITMAX;m++) {
		em=(double) m;
		tem=em+em;
		d=em*(b-em)*x/((qam+tem)*(a+tem));
		ap=az+d*am;
		bp=bz+d*bm;
		d = -(a+em)*(qab+em)*x/((qap+tem)*(a+tem));
		app=ap+d*az;
		bpp=bp+d*bz;
		aold=az;
		am=ap/bpp;
		bm=bp/bpp;
		az=app/bpp;
		bz=1.0;
		if (fabs(az-aold) < (EPS*fabs(az))) return az;
	}
	fprintf(stderr,"a or b too big, or ITMAX too small in BETACF");
}

#undef ITMAX
#undef EPS

double gammln(xx)
double xx;
{
	double x,tmp,ser;
	static double cof[6]={76.18009173,-86.50532033,24.01409822,
		-1.231739516,0.120858003e-2,-0.536382e-5};
	int j;

	x=xx-1.0;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.0;
	for (j=0;j<=5;j++) {
		x += 1.0;
		ser += cof[j]/x;
	}
	return -tmp+log(2.50662827465*ser);
}

double betai(a,b,x)
double a,b,x;
{
	double bt;
	double gammln(),betacf();

	if (x < 0.0 || x > 1.0) fprintf(stderr,"Bad x in routine BETAI");
	if (x == 0.0 || x == 1.0) bt=0.0;
	else
		bt=exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.0-x));
	if (x < (a+1.0)/(a+b+2.0))
		return bt*betacf(a,b,x)/a;
	else
		return 1.0-bt*betacf(b,a,1.0-x)/b;
}

void avevar(data,n,ave,svar)
double data[],*ave,*svar;
int n;
{
	int j;
	double s;

	*ave=(*svar)=0.0;
	for (j=1;j<=n;j++) *ave += data[j];
	*ave /= n;
	for (j=1;j<=n;j++) {
		s=data[j]-(*ave);
		*svar += s*s;
	}
	*svar /= (n-1);
}

void ttest(data1,n1,data2,n2,t,prob)
int n1,n2;
double data1[],data2[],*t,*prob;
{
	double var1,var2,svar,df,ave1,ave2;
	void avevar();
	double betai();

	avevar(data1,n1,&ave1,&var1);
	avevar(data2,n2,&ave2,&var2);
	df=n1+n2-2;
	svar=((n1-1)*var1+(n2-1)*var2)/df;
	*t=(ave1-ave2)/sqrt(svar*(1.0/n1+1.0/n2));
	*prob=betai(0.5*df,0.5,df/(df+(*t)*(*t)));
}


void tptest(data1,data2,n,t,prob)
double data1[],data2[],*t,*prob;
int n;
{
	int j;
	double var1,var2,ave1,ave2,sd,df,cov=0.0;
	void avevar();
	double betai();

	avevar(data1,n,&ave1,&var1);
	avevar(data2,n,&ave2,&var2);
	for (j=1;j<=n;j++)
		cov += (data1[j]-ave1)*(data2[j]-ave2);
	cov /= df=n-1;
	sd=sqrt((var1+var2-2.0*cov)/n);
	*t=(ave1-ave2)/sd;
	*prob=betai(0.5*df,0.5,df/(df+(*t)*(*t)));
}

main(int argc, char *argv[])
{
  int  	i, j, k;
  FILE 	*fp;
  char 	s[100];
  int	val, val2;
  int	count;
  double ksum, lsum, kavg, lavg;
  double dsum, sumsqr, sd;
  double davg;
  double t, prob;
  double ave,adev,sdev,svar,skew,curt;
  int n;
  /* read in data points */

  count = 1;
  while(!feof(stdin))
  {
    fscanf(stdin,"%lf %lf\n",&(kpoints[count]),&(lpoints[count]));
    printf("%lf %lf\n",kpoints[count],lpoints[count]);
    count ++;
  }
      
  count = count - 1;
  n = count;
  dsum = 0.0;
  sumsqr = 0.0;
  ksum = lsum = 0.0;
  /* for (i=1; i<=count; ++i)
  {
    printf("%lf, %lf\n",kpoints[i], lpoints[i]);
  }
  */
  for (i=1; i<=count; ++i)
  {
    fprintf(stderr,"k(%d) = %lf, l(%d) = %lf ",i, kpoints[i],i , lpoints[i]);
    ksum += kpoints[i];
    lsum += lpoints[i];
    diff[i] = kpoints[i] - lpoints[i];
    dsum += diff[i];
    sumsqr += diff[i]*diff[i]; 
    fprintf(stderr,"diff = %lf, diff^2 = %lf\n",diff[i],diff[i]*diff[i]);
  }
  kavg = ksum/(double)count;
  lavg = lsum/(double)count;
  davg = dsum/(double)count;
  sd = sqrt( ((double)count*sumsqr - dsum*dsum)
	    /((double)count*(double)(count-1)));

  fprintf(stderr,"n = %d\n",count);
  fprintf(stderr,"d_bar = %lf, d_s = %lf\n",davg,sd);
  fprintf(stderr,"Avg_k = %lf, Avg_l = %lf, Avg_k-Avg_l = %lf\n", kavg, lavg, kavg-lavg);
  fprintf(stderr,"Std Deviation of difference between kohonen and lissom = %lf\n",sd);
  fprintf(stderr,"%lf < U_d < %lf with alpha = 0.02 (98%% confidence interval)\n",
		davg - (1.645)*sd/sqrt((double)count),
		davg + (1.645)*sd/sqrt((double)count));


  tptest(kpoints, lpoints, count, &t, &prob);
  fprintf(stderr,"TPtest result : t = %lf,  prob = %lf\n",t,prob);
  ttest(kpoints, count, lpoints, count, &t, &prob);
  fprintf(stderr,"Ttest result : t = %lf,  prob = %lf\n",t,prob);

  fprintf(stderr,"\n\n Stat report \n");
  moment(kpoints,n,&ave,&adev,&sdev,&svar,&skew,&curt);
  fprintf(stderr,"k : n = %d, ave = %lf, adev = %lf, sdev = %lf, svar = %lf \n",
	n, ave, adev, sdev, svar);

  moment(lpoints,n,&ave,&adev,&sdev,&svar,&skew,&curt);
  fprintf(stderr,"\n\nl : n = %d, ave = %lf, adev = %lf, sdev = %lf, svar = %lf \n",
	n, ave, adev, sdev, svar);
}
  
  

