% NOISEONF - Creates 1/f spectrum noise images.
%
% Function to create noise images having 1/f amplitude spectum properties
%
% Usage: im = noiseonf(size, factor)
%
%        size   - size of image to produce
%        factor - controls spectrum = 1/(f^factor)
%
%        factor = 0   - raw Gaussian noise image
%               = 1   - gives the 1/f `standard' drop-off for `natural' images
%               = 1.5 - seems to give the most intersting `cloud patterns'
%               = 2 or greater - produces `blobby' images
% PK 3-12-96
%
%-- Comment added by Yoonsuck Choe Sun Dec 29 20:09:14 CST 2002
% M-file Author: Peter Kovesi
% URL: http://www.cs.uwa.edu.au/~pk/Research/MatlabFns/
% Original copyright: unknown
% Modifications by yschoe: made the function compatible with GNU/octave

function im = noiseonf(size, factor)

rows = size;
cols = size;
im = randn(size,size);  % Random Gaussian noise, mean 0, std dev 1.

imfft = fft2(im);                   % Take fft of image.
imfft = fftshift(imfft);            % Shift 0 frequency to the middle.
mag = abs(imfft);                   % Get magnitude
phase = angle(imfft);               % and phase.

% Create two matrices, x and y. All elements of x have a value equal to its 
% x coordinate relative to the centre, elements of y have values equal to 
% their y coordinate relative to the centre.  From these two matrices produce
% a radius matrix that gives distances from the middle

x = ones(rows,1) * (-cols/2 : (cols/2 - 1)); 
y = (-rows/2 : (rows/2 - 1))' * ones(1,cols);
radius = sqrt(x.^2 + y.^2);         % Matrix values contain radius from centre.
radius(rows/2+1,cols/2+1) = 1;      % .. avoid division by zero.

filter = 1./(radius.^factor);       % Construct the filter.

% Reconstruct fft of noise image, but now with the specified amplitude spectrum
%newfft = (mag .* filter) .* exp(i*phase);
newfft =  filter .* exp(i*phase);
im = real(ifft2(fftshift(newfft))); % Invert to obtain final noise image

caption = sprintf('noise with 1/(f^%2.1f) amplitude spectrum',factor);
imagesc(im); axis('equal'), axis('off'), title(caption);
