#!/usr/bin/wish
#----------------------------------------------------------------------------
# Config
#----------------------------------------------------------------------------

set bgColor "\#ffffff"
set winName(0) "Input"
set winName(1) "Orientation Map"
set winName(2) "Spikes"
set winName(3) "Excitatory Contrib"

set bg(0) "\#ee8888"
set bg(1) "\#88ee99"
set bg(2) "\#88ee99"
set bg(3) "\#88ee99"

set labelFont "-adobe-courier-bold-r-normal--25-180-100-100-m-150-iso8859-1"

option add *background $bgColor

set num 0
set sleepSec 0 	; # in ms
set dirPrefix "./0-degrees"
set stopFlag 0
set pauseFlag 0
#----------------------------------------------------------------------------
# END config
#----------------------------------------------------------------------------

. configure -background $bgColor

#----------------------------------------------------------------------------
# testMatImg : 
#----------------------------------------------------------------------------
proc testMatImg { imgName fileprefix filenum places {zoom 3} } {

  global dirPrefix
  
  set num [format "%0${places}d" $filenum]
  if { $filenum >= 0 } {
  	set filename "${fileprefix}.${num}.ppm"
  } else {
	set filename "${fileprefix}"
  }

  set filename "$dirPrefix/$filename"
  destroy $imgName
  destroy tmpImg
  image create photo tmpImg -format ppm -file  $filename
  image create photo $imgName
  $imgName copy tmpImg -zoom $zoom $zoom
}

#----------------------------------------------------------------------------
# run : run demo for current directory
#----------------------------------------------------------------------------
proc run { zoomBase } {

	global num dirPrefix sleepSec stopFlag pauseFlag

	if { $dirPrefix == "./4input" } {
		set start 0 
		set end 599
		set zoom [expr 5 * $zoomBase];
	} else {
		set start 40010 
		set end 40600
		set iterOffset 0;
		set zoom $zoomBase;
  	}

	set num 0
	testMatImg MAT0 "output.00000.ppm" -1 5 $zoomBase
	testMatImg MAT1 "oct24-1.040000.map2-0.or.ppm" -1 5 $zoom
	for {set i $start} {$i<$end} {incr i} {
	  testMatImg MAT2 "spikeact_q" $i 5 $zoom
	  testMatImg MAT3 "net_exc_input" $i 5 $zoom
	  incr num
	  update
	  after $sleepSec
	  if { $stopFlag == 1 } {
		set stopFlag 0
		break;
	  }
	  if { $pauseFlag == 1} {
		vwait pauseFlag;
  	  }
	}
}
#============================================================================
# MAIN
#============================================================================

#----------------------------------------
# 0. arguments
#----------------------------------------
if { $argc == 1} {
	set zoom [lindex $argv 0]
} else {
	set zoom 5
}

#----------------------------------------
# 1. load file list
#----------------------------------------

#----------------------------------------
# 2. Create ppm display windows
#----------------------------------------
#--------------------
# 2.1 Frame for holding ppm windows 
#--------------------
frame .winFrame -background $bgColor; 
pack .winFrame -side top -padx 5 -pady 5

#--------------------
# 2.2 Create ppm windows and associated images
#--------------------
set prefix ".winFrame"
set num 0
for {set i 0} {$i<4} {incr i} {
  image create photo MAT$i 

  set subframe "$prefix.frame$i"
  frame $subframe; pack $subframe -side left -padx 5

  # Image
  label $subframe.mat$i -image MAT$i -relief sunken -borderwidth 2
  pack $subframe.mat$i -side top

  # Name
  label $subframe.name$i -text $winName($i) -relief sunken -borderwidth 1 \
	-background $bg($i) -font $labelFont
  pack $subframe.name$i -side top -fill x

}

testMatImg MAT0 "output.00000.ppm" -1 5 $zoom
testMatImg MAT1 "oct24-1.040000.map2-0.or.ppm" -1 5 $zoom
testMatImg MAT2 "spikeact_q" 40010 5 $zoom
testMatImg MAT3 "net_exc_input" 40010 5 $zoom

update

#--------------------
# 2.3 Iteration count
#--------------------
label .num -textvariable num; pack .num -side top

#--------------------
# 2.4 Until buttons
#--------------------

frame .util -relief ridge -borderwidth 2 -background $bgColor
pack .util -side top -padx 10 -pady 10 

#----------
# Listbox
#----------

# Menu Button
menubutton .util.list -text "Demo Selection" -direction below \
	-menu .util.list.choice -border 2 -relief raised
pack .util.list -side left  -padx 10

# Menu items
menu .util.list.choice 

.util.list.choice add radiobutton -variable dirPrefix -value \
	"./0-degrees" -label "0 Degree" \
	-command {testMatImg MAT0 "output.00000.ppm" -1 5 $zoom; update; }
.util.list.choice add radiobutton -variable dirPrefix -value \
	"./90-degrees" -label "90 Degrees" \
	-command {testMatImg MAT0 "output.00000.ppm" -1 5 $zoom; update; }
.util.list.choice add radiobutton -variable dirPrefix -value \
	"./2-contours" -label "Segmentation" \
	-command {testMatImg MAT0 "output.00000.ppm" -1 5 $zoom; update; }
.util.list.choice add radiobutton -variable dirPrefix -value \
	"./filling-in" -label "Completion" \
	-command {testMatImg MAT0 "output.00000.ppm" -1 5 $zoom; update; }
.util.list.choice add radiobutton -variable dirPrefix -value \
	"./4input" -label "4 Input" \
	-command {testMatImg MAT0 "output.00000.ppm" -1 5 $zoom; update; }

#----------
# Scrollbar for delay between frames
#----------
label .util.sbname -text "Time Delay (ms):" 
pack .util.sbname -side left -padx 5
scale .util.delay -orient horizontal 
pack .util.delay -side left -padx 5 -fill x
.util.delay configure -from 0 -to 2000 -variable sleepSec  -resolution 100

#----------
# Buttons
#----------
set pausedFlag 0

#---
# RUN
#---
button .util.run -text "Run" -command { set pauseFlag 0;  run $zoom; }; 
pack .util.run -side left

#---
# PAUSE
#---
button .util.pause -text "Pause/Resume" -command \
	{ set pauseFlag [expr ($pauseFlag + 1)%2]; 
	  if {$pauseFlag == 0} {
		set dummyFlag 0
		.util.pauseStat deselect 
  	  } else {
		set dummyFlag 1
		.util.pauseStat select
	  }
	  update
        }
pack .util.pause -side left

#---
# Stat
#---
set dummyFlag 0
radiobutton .util.pauseStat -variable dummyFlag 
pack .util.pauseStat -side left

#---
# Stop
#---
button .util.stop -text "Stop(Reset)" -command { set stopFlag 1; }; 
pack .util.stop -side left

#---
# Quit
#---
button .util.q -text "Quit" -command exit ; 
pack .util.q -side left -padx 10 

wm title . "SLISSOM Demo"
